#!/bin/env python
import os
import time
import ConfigParser
import string

class Archive:
    """Reprsente une archive de partition"""
    def __init__(self):
        self.name = None
        self.comment = None
        self.creationDate = None
        self.lastRestoreDate = None
        self.file = None
        self.filesize = None
        self.dev = None
        self.default = None
        self.type = None
        
    def make(self):
        """Fabrique le fichier archive"""
        self.makereal()
        self.creationDate = self.now()
        ## Calcule la taille du fichier archive en Mo
        self.filesize = os.stat(self.file)[6] / (1024 * 1024)

    def restore(self):
        """Restaure la partition archive"""
        self.restorereal()
        time.sleep(5)
        self.lastRestoreDate = self.now()

    def delete(self):
        """Supprime le fichier d'archive du disque"""
        os.remove(self.file)

    def makereal(self):
        """A sous-classer. Fabrique l'archive selon une mthode."""
        print "archive vide"
        
    def restorereal(self):
        """A sous-classer. Restaure l'archive selon une mthode."""
        print "restaure vide"

    def __repr__(self):
        """Pour l'impression des archives"""
        return self.name

    def now(self):
        """Renvoie la date actuelle"""
        now = time.localtime(time.time())
        return time.strftime("%c")


    def halt(self):
        """Arrte l'ordinateur"""
        os.system("halt")

    def reboot(self):
        """Reboote l'ordinateur"""
        os.system("reboot")

    def df(self):
        """Renvoie une liste:
        (size,  mntpoint, occupe)
        du priphrique self.dev"""
        dflines = os.popen("df -h %s" % self.dev).readlines()
        #(dev, tail, util, disp, util, mntpoint) = dflines[1]
        #return (tail, util, mntpoint)
        fields = string.split(dflines[1])
        return (fields[1], fields[5], fields[2])
        
class Archive_dd(Archive):
    """Archive selon la copie par blocs dd"""
    def __init__(self):
        Archive.__init__(self)
        self.type = 'dd'
        
    def makereal(self):
        """Fabrique le fichier archive avec dd"""
        print "##### Cration archive dd #####"
        print "dd if=%s of=%s" % (self.dev, self.file)
        os.system("dd if=%s of=%s" % (self.dev, self.file))
        print "Cration de l'archive termine"
        
    def restore(self):
        """Restaure l'archive"""
        print "##### Restauration dd #####"
        print "dd if=%s of=%s" % (self.file, self.dev)
        os.system("dd if=%s of=%s" % (self.file, self.dev))
        print "restauration termine"

class Archive_ddzip(Archive):
    """Archive selon la copie par blocs"""
    def __init__(self):
        Archive.__init__(self)
        self.type = 'dd_zip'
        
    def makereal(self):
        """Fabrique le fichier archive avec dd"""
        print "##### Cration archive ddzip #####"
        mntpoint = self.df()[1]
        print "fichier dummy %s/dummy" % mntpoint
        os.system("dd if=/dev/zero of=%s/dummy" % mntpoint)
        print "fichier dummy cree. On l'efface"
        os.system("rm %s/dummy" % mntpoint)
        print "Cration de l'archive dd zipe" 
        os.system("dd if=%s | gzip | dd of=%s" % (self.dev, self.file))
        print "Archive termine"
        
    def restorereal(self): 
        """Restaure l'archive"""
        print "##### Restauration ddzip #####"
        print "restauration zcat %s | dd of=%s" % (self.file, self.dev)
        os.system("zcat %s | dd of=%s" % (self.file, self.dev))
        print "restauration termine"
        
class Archive_ddsector(Archive):
    """copie dd d'un unique secteur dd bs=512 count=1"""
    def __init__(self):
        Archive.__init__(self)
        self.type = 'dd_sector'
        
    def makereal(self):
        """Fabrique la copie d'un secteur"""
        print "##### Cration archive ddsector #####"
        print "dd if=%s of=%s bs=512 cournt=1" % (self.dev, self.file)
        os.system("dd if=%s of=%s bs=512 count=1" % (self.dev, self.file))
        print "##### Cration archive termine #####"
        
    def restorereal(self):
        """Restaure le secteur"""
        print "##### Restauration archive ddsector #####"
        print "dd if=%s of=%s bs=512 count=1" % (self.file, self.dev)
        os.system("dd if=%s of=%s bs=512 count=1" % (self.file, self.dev))
        print "##### Restauration ddsector termine #####"

class Archive_tgz(Archive):
    """archive tgz"""
    def __init__(self):
        Archive.__init__(self)
        self.type = 'tgz'
        
    def makereal(self):
        """Construit l'archive  partir du point de montage de dev"""
        print "##### Cration archive tgz #####"
        mntpoint = self.df()[1]
        os.chdir('/')
        print "cd /, tar czf %s %s" % (self.file, mntpoint)
        os.system("tar czf %s %s" % (self.file, mntpoint))
        print "##### Cration archive tgz termine #####"
        
    def restorereal(self):
        """Restaure l'archive tgz"""
        print "##### Restauration archive tgz #####"
        mntpoint = self.df()[1]
        os.chdir('/')
        print "cd / , tar xzf %s" % self.file
        os.system("tar xzf %s" % self.file)
        print "##### Restauration tgz termine #####"

#####################################################################
#####################################################################

class ArchiveList:
    """Manipulation d'une liste d'archives, synchronise avec
    le fichier de configuration"""
    def __init__(self):
        self.configFile = "/sauvegarde/restaure.cfg"
        self.archives = {}
        self.readConfig()

        
    def makeNewArchive(self, type, name, comment,
                       file, dev, default=0):
        """Cration d'une nouvelle archive.
        type = dd|dd_zip|tgz|dd_sector"""
        if type == 'dd':
            archive = Archive_dd()
        elif type == 'dd_zip':
            archive = Archive_ddzip()
        elif type == 'tgz':
            archive = Archive_tgz()
        elif type == 'dd_sector':
            archive = Archive_ddsector()
        else:
            print "type d'archive inconnu"
        archive.name = name
        archive.comment = comment
        archive.file = file
        archive.dev = dev
        archive.default = default
        archive.make()
        self.archives[name] = archive
        self.writeConfig()

    def setDefault(self, name, default):
        """Met l'archive de nom name  dfaut"""
        self.archives[name].default = default
        self.writeConfig()
        
    def delete(self, name):
        """Supprime l'archive nomme name"""
        print "suppression de l'archive %s" % name
        archive = self.archives[name]
        archive.delete()
        del self.archives[name]
        self.writeConfig()

    def restoreDefaults(self):
        """Restaure les archives par dfaut"""
        for name, archive in self.archives.items():
            if archive.default == '1':
                archive.restore()

    def setDefault(self, name, defaut):
        """Met la variable defaut de l'archive de nom name  la valeur
        passe dans defaut"""
        self.archives[name].default = defaut
        self.writeConfig()
        
    def readConfig(self):
        """Lit le fichier des archives et met  jour archives"""
        config = ConfigParser.ConfigParser()
        config.read(self.configFile)
        for section in config.sections():
            type = config.get(section, 'type')
            if type == 'dd':
                archive = Archive_dd()
            elif type == 'dd_zip':
                archive = Archive_ddzip()
            elif type == 'dd_sector':
                archive = Archive_ddsector()
            elif type == 'tgz':
                archive = Archive_tgz()
            archive.name = section
            archive.comment = config.get(section, 'comment')
            archive.file = config.get(section, 'file')
            archive.filesize = config.get(section, 'filesize')
            archive.dev = config.get(section, 'dev')
            archive.creationDate = config.get(section, 'creationDate')
            archive.lastRestoreDate = config.get(section, 'lastRestoreDate')
            archive.default = config.get(section, 'default')
            self.archives[section] = archive
            print "readConfig : %s" % section
            
    def writeConfig(self):
        """Met  jour le fichier des archives"""
        config = ConfigParser.ConfigParser()
        for name, archive in self.archives.items():
            config.add_section(name)
            config.set(name, 'type', archive.type)
            config.set(name, 'comment', archive.comment)
            config.set(name, 'file', archive.file)
            config.set(name, 'filesize', archive.filesize)
            config.set(name, 'dev', archive.dev)
            config.set(name, 'creationDate', archive.creationDate)
            config.set(name, 'lastRestoreDate', archive.lastRestoreDate)
            config.set(name, 'default', archive.default)
        file = open(self.configFile, 'w')
        config.write(file)
        file.close()
        print "fichier conf:  %s mis  jour" % self.configFile
    
    
if __name__ == '__main__':
    archivelist = ArchiveList()
    archivelist.restoreDefaults()
    os.system("reboot")
